/'
 ' Project Publisher - Installer Creator Addin for FbEdit, RadASM and MasmEdit
 '
 ' Copyright (C) 2010 Arnel A. Borja (Galeon)
 '
 ' This program is free software: you can redistribute it and/or modify it under 
 ' the terms of the GNU General Public License as published by the Free Software 
 ' Foundation, either version 3 of the License, or (at your option) any later 
 ' version.
 '
 ' This program is distributed in the hope that it will be useful, but WITHOUT 
 ' ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 ' FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more 
 ' details.
 '
 ' You should have received a copy of the GNU General Public License along with 
 ' this program.  If not, see <http://www.gnu.org/licenses/>.
 '
 '/

#If Not Defined( FBE_ADDIN ) And _
	Not Defined( RA2_ADDIN ) And _
	Not Defined( RA3_ADDIN ) And _
	Not Defined( ME_ADDIN )
#Define FBE_ADDIN
#EndIf

#Include Once "windows.bi"
#Include Once "win/commctrl.bi"
#Include Once "win/commdlg.bi"
#Include Once "win/shellapi.bi"

#Include Once "Inc/FbEdit.bi"
#Include Once "Inc/RadASM2.bi"
#Include Once "Inc/RadASM3.bi"
#Include Once "Inc/MasmEd.bi"

#Include Once "ProjPub.bi"
#Include Once "Wizard.bi"
#Include Once "Settings.bi"
#Include Once "FileFuncs.bi"
#Include Once "MsgBox.bi"
#Include Once "Welcome.bi"
#Include Once "ProjType.bi"
#Include Once "ProjInfo.bi"
#Include Once "ProjWeb.bi"
#Include Once "License.bi"
#Include Once "MultiUser.bi"
#Include Once "InstallFolder.bi"
#Include Once "Files.bi"
#Include Once "Mui.bi"
#Include Once "RemDrvInst.bi"
#Include Once "Shortcuts.bi"
#Include Once "LangDll.bi"
#Include Once "FileAssoc.bi"
#Include Once "Uninst.bi"
#Include Once "Compressor.bi"
#Include Once "Preview.bi"
#Include Once "Publish.bi"
#Include Once "Finish.bi"
#Include Once "Failed.bi"
#Include Once "Resource.bi"

#If Defined( FBE_ADDIN )

' For FbEdit
' Returns info on what messages the addin hooks into (in an ADDINHOOKS type).
Function InstallDll Cdecl Alias "InstallDll" ( _
	ByVal hWin As HWND, _
	ByVal hInst As HINSTANCE _
	) As FBEADDINHOOKS Ptr Export

	Dim lf As LOGFONT
	Dim hDC As HDC

	' Set current IDE
	CurIDE = IDE_FBEDIT

	' The dll's instance
	hInstance = hInst

	' Get pointer to ADDINHANDLES, ADDINDATA and ADDINFUNCTIONS
	lpFbEHandles = Cast( FBEADDINHANDLES Ptr, SendMessage( hWin, FBEAIM_GETHANDLES, 0,0 ) )
	lpFbEData = Cast( FBEADDINDATA Ptr, SendMessage( hWin, FBEAIM_GETDATA, 0, 0 ) )
	lpFbEFunctions = Cast( FBEADDINFUNCTIONS Ptr, SendMessage( hWin, FBEAIM_GETFUNCTIONS, 0, 0 ) )

	' Get IDE path
	IDEPath = lpFbEData->AppPath

	'' Fill a MENUITEMINFO structure
	Dim mii As MENUITEMINFO
	mii.cbSize = SizeOf( MENUITEMINFO )
	mii.fMask  = MIIM_SUBMENU

	'' Get handle to 'Make' popup
	GetMenuItemInfo( lpFbEHandles->hmenu, IDM_FBEMAKE, FALSE, @mii )
	MakeSubMenu = mii.hSubMenu

	'' First get a menu id
	IDM_PUBLISH = SendMessage( hWin, FBEAIM_GETMENUID, 0, 0 )
	'' Add our menu item to Tools menu
	AppendMenu( MakeSubMenu, MF_STRING Or MF_GRAYED, IDM_PUBLISH, GetTranslateString( "10000", "Publish" ) & !"\tCtrl+Shift+Alt+F5" )
	AddAccelerator( FVIRTKEY Or FNOINVERT Or FCONTROL Or FSHIFT Or FALT, VK_F5, IDM_PUBLISH )

	'' Get handle to 'Options' popup
	GetMenuItemInfo( lpFbEHandles->hmenu, IDM_FBEOPTIONS, FALSE, @mii )

	'' Get another menu id
	IDM_PUBLISHOPTIONS = SendMessage( hWin, FBEAIM_GETMENUID, 0, 0 )
	'' Add our menu item to Tools menu
	AppendMenu( mii.hSubMenu, MF_STRING, IDM_PUBLISHOPTIONS, GetTranslateString( "10001", "Project Publisher" ) )

	' Load settings
	LoadSettingsIni( @lpFbEData->IniFile )

	' Load license, graphics and languages files
	LoadLicenses( IDEPath + "\ProjPub\Licenses" )
	LoadGraphics( IDEPath + "\ProjPub\Graphics" )
	LoadLanguages( IDEPath + "\ProjPub\Languages" )

	' Messages this addin will hook into
	FbEHooks.hook1 = FBEAIH_COMMAND Or FBEAIH_CLOSE Or FBEAIH_MENUENABLE Or FBEAIH_PROJECTOPEN Or FBEAIH_PROJECTCLOSE
	FbEHooks.hook2 = NULL
	FbEHooks.hook3 = NULL
	FbEHooks.hook4 = NULL

	Return @FbEHooks

End Function

' FbEdit calls this function for every addin message that this addin is hooked into.
' Returning TRUE will prevent FbEdit and other addins from processing the message.
Function DllFunction Cdecl Alias "DllFunction" ( _
	ByVal hWin As HWND, _
	ByVal uMsg As UINT, _
	ByVal wParam As WPARAM, _
	ByVal lParam As LPARAM _
	) As BOOL Export

	Select Case uMsg
		Case FBEAIM_COMMAND
			Select Case LoWord( wParam )
				Case IDM_PUBLISH
					ShowProjectPublishWizard( hWin )
					'
				Case IDM_PUBLISHOPTIONS
					DialogBox( hInstance, MAKEINTRESOURCE( IDD_DLGSETTINGS ), hWin, @SettingsDialogProc )
					'
			End Select
			'
		Case FBEAIM_PROJECTOPEN
			CurProjectFile = lpFbEData->ProjectFile
			CurProjectPath = lpFbEData->ProjectPath
			LoadProjectSettings( @CurProjectFile )
			'
		Case FBEAIM_PROJECTCLOSE
			SaveProjectSettings( @CurProjectFile )
			RemoveAllFolders( @ProjectSettings.Folders )
			RemoveAllFileTypes( @ProjectSettings.FileAssociations )
			CurProjectFile = ""
			CurProjectPath = ""
			'
		Case FBEAIM_MENUENABLE
			Dim uEnable As Integer = MF_BYCOMMAND
			If CurProjectFile <> "" Then
				uEnable Or= MF_ENABLED
			Else
				uEnable Or= MF_GRAYED
			End If
			EnableMenuItem( MakeSubMenu, IDM_PUBLISH, uEnable )
			'
		Case FBEAIM_CLOSE
			DeleteMenu( lpFbEHandles->hMenu, IDM_PUBLISH, MF_BYCOMMAND )
			DeleteMenu( lpFbEHandles->hMenu, IDM_PUBLISHOPTIONS, MF_BYCOMMAND )
			If WelcomeMsgFont Then DeleteObject( WelcomeMsgFont )
			If HeaderTitleFont Then DeleteObject( HeaderTitleFont )
			FreeLicenses
			FreeGraphics
			SaveSettingsIni( lpFbEData->IniFile )
			'
	End Select

	Return FALSE

End Function

#ElseIf Defined( RA2_ADDIN )

' For RadASM 2.x
' Returns info on what messages the addin hooks into (in an ADDINHOOKS structure).
Function InstallDllEx CDECL alias "InstallDllEx" ( _
	ByVal hWin as HWND, _
	ByVal fOptions as Long, _
	ByVal hInst as HINSTANCE _
	) As RA2ADDINHOOKS Ptr Export

	' Set current IDE
	CurIDE = IDE_RADASM2

	' The dll's instance
	hInstance = hInst

	' Get pointer to ADDINHANDLES, ADDINDATA and ADDINPROCS
	lpRa2Handles = Cast( RA2ADDINHANDLES Ptr, SendMessage( hWin, RA2AIM_GETHANDLES, 0,0 ) )
	lpRa2Data = Cast( RA2ADDINDATA Ptr, SendMessage( hWin, RA2AIM_GETDATA, 0, 0 ) )
	lpRa2Functions = Cast( RA2ADDINPROCS Ptr, SendMessage( hWin, RA2AIM_GETPROCS, 0, 0 ) )

	' Get IDE path
	IDEPath = *lpRa2Data->lpApp

	'' Fill a MENUITEMINFO structure 
	Dim mii As MENUITEMINFO
	mii.cbSize = SizeOf( MENUITEMINFO )
	mii.fMask  = MIIM_SUBMENU

	'' Get handle to 'Make' popup
	GetMenuItemInfo( lpRa2Handles->hmenu, IDM_RA2MAKE, FALSE, @mii )
	MakeSubMenu = mii.hSubMenu

	'' First get a menu id
	IDM_PUBLISH = SendMessage( hWin, RA2AIM_GETMENUID, 0, 0 )
	'' Add our accelerator to menu
	'' The menu will not be added since it will be removed when a project is loaded
	AddAccelerator( FVIRTKEY Or FNOINVERT Or FCONTROL Or FSHIFT Or FALT, VK_F5, IDM_PUBLISH )
	
	'' Get handle to 'Options' popup
	GetMenuItemInfo( lpRa2Handles->hmenu, IDM_RA2OPTIONS, FALSE, @mii )
	
	'' Get another menu id
	IDM_PUBLISHOPTIONS = SendMessage( hWin, RA2AIM_GETMENUID, 0, 0 )
	'' Add our menu item to Tools menu

	AppendMenu( mii.hSubMenu, MF_STRING, IDM_PUBLISHOPTIONS, GetTranslateString( "10001", "Project Publisher" ) )

	' Load settings
	LoadSettingsIni( lpRa2Data->lpIniFile )

	' Load license, graphics and languages files
	LoadLicenses( IDEPath + "\ProjPub\Licenses" )
	LoadGraphics( IDEPath + "\ProjPub\Graphics" )
	LoadLanguages( IDEPath + "\ProjPub\Languages" )

	' Messages this addin will hook into
	Ra2Hooks.fHook1 = RA2AIH_COMMAND Or RA2AIH_CLOSE Or RA2AIH_PROJECTOPENED Or RA2AIH_PROJECTCLOSE
	Ra2Hooks.fHook2 = NULL
	Ra2Hooks.fHook3 = NULL

	Return @Ra2Hooks

End Function

' RadASM 2.x calls this function for every addin message that this addin is hooked into.
Function DllProc Cdecl Alias "DllProc" ( _
	ByVal hWin As HWND, _
	ByVal uMsg As UINT, _
	ByVal wParam As WPARAM, _
	ByVal lParam As LPARAM _
	) As BOOL Export

	Select Case uMsg
		Case RA2AIM_COMMAND
			Select Case LoWord( wParam )
				Case IDM_PUBLISH
					ShowProjectPublishWizard( hWin )
					'
				Case IDM_PUBLISHOPTIONS
					DialogBox( hInstance, MAKEINTRESOURCE( IDD_DLGSETTINGS ), hWin, @SettingsDialogProc )
					'
			End Select
			'
		Case RA2AIM_PROJECTOPENED
			CurProjectFile = *lpRa2Data->lpProject
			CurProjectPath = *lpRa2Data->lpProjectPath
			' Remove ending '\'
			CurProjectPath = Left( CurProjectPath, Len( CurProjectPath ) - 1 )
			LoadProjectSettings( @CurProjectFile )
			AppendMenu( MakeSubMenu, MF_STRING Or MF_ENABLED, IDM_PUBLISH, GetTranslateString( "10000", "Publish" ) & !"\tCtrl+Shift+Alt+F5" )
			'
		Case RA2AIM_PROJECTCLOSE
			SaveProjectSettings( @CurProjectFile )
			RemoveAllFolders( @ProjectSettings.Folders )
			RemoveAllFileTypes( @ProjectSettings.FileAssociations )
			CurProjectFile = ""
			CurProjectPath = ""
			'
		Case RA2AIM_CLOSE
			DeleteMenu( lpRa2Handles->hMenu, IDM_PUBLISH, MF_BYCOMMAND )
			DeleteMenu( lpRa2Handles->hMenu, IDM_PUBLISHOPTIONS, MF_BYCOMMAND )
			If WelcomeMsgFont Then DeleteObject( WelcomeMsgFont )
			If HeaderTitleFont Then DeleteObject( HeaderTitleFont )
			FreeLicenses
			FreeGraphics
			SaveSettingsIni( *lpRa2Data->lpIniFile )
			'
	End Select

	Return FALSE

End Function

#ElseIf Defined( RA3_ADDIN )

' For RadASM 3.x
' Returns info on what messages the addin hooks into (in an ADDINHOOKS structure).
Function InstallAddin CDECL alias "InstallAddin" ( _
	ByVal hWin as HWND _
	) As RA3HOOK Ptr Export

	' Set current IDE
	CurIDE = IDE_RADASM3

	' The dll's instance
	hInstance = GetModuleHandle( ExePath & "\Addins\ProjPub.dll" )
	If hInstance = NULL Then
		ErrorBox( hWin, GetTranslateString( "5009", "Can't find instance of addin." ) )
		Return @Ra3Hooks
	End If

	' Get pointer to ADDINHANDLES, ADDINDATA and ADDINPROCS
	lpRa3Handles = Cast( RA3ADDINHANDLES Ptr, SendMessage( hWin, RA3AIM_GETHANDLES, 0,0 ) )
	lpRa3Data = Cast( RA3ADDINDATA Ptr, SendMessage( hWin, RA3AIM_GETDATA, 0, 0 ) )
	lpRa3Functions = Cast( RA3ADDINPROCS Ptr, SendMessage( hWin, RA3AIM_GETPROCS, 0, 0 ) )

	' Get IDE path
	' TODO: Doesn't work in 3.0.0.4
	'IDEPath = lpRa3Data->szAppPath
	IDEPath = ExePath

	'' Fill a MENUITEMINFO structure 
	Dim mii As MENUITEMINFO
	mii.cbSize = SizeOf( MENUITEMINFO )
	mii.fMask  = MIIM_SUBMENU

	'' Get handle to 'Make' popup
	GetMenuItemInfo( lpRa3Handles->hMenu, RA3IDM_MAKE, FALSE, @mii )
	MakeSubMenu = mii.hSubMenu

	'' First get a menu id
	IDM_PUBLISH = SendMessage( hWin, RA3AIM_GETMENUID, 0, 0 )
	'' Add our menu item to Tools menu
	AppendMenu( MakeSubMenu, MF_STRING Or MF_GRAYED, IDM_PUBLISH, GetTranslateString( "10000", "Publish" ) & !"\tCtrl+Shift+Alt+F5" )
	AddAccelerator( FVIRTKEY Or FNOINVERT Or FCONTROL Or FSHIFT Or FALT, VK_F5, IDM_PUBLISH )
	
	'' Get handle to 'Options' popup
	GetMenuItemInfo( lpRa3Handles->hMenu, RA3IDM_OPTION, FALSE, @mii )
	
	'' Get another menu id
	IDM_PUBLISHOPTIONS = SendMessage( hWin, RA3AIM_GETMENUID, 0, 0 )
	'' Add our menu item to Tools menu
	AppendMenu( mii.hSubMenu, MF_STRING, IDM_PUBLISHOPTIONS, GetTranslateString( "10001", "Project Publisher" ) )

	' Load settings
	LoadSettingsIni( lpRa3Data->szRadASMIni )

	' Load license, graphics and languages files
	LoadLicenses( IDEPath + "\ProjPub\Licenses" )
	LoadGraphics( IDEPath + "\ProjPub\Graphics" )
	LoadLanguages( IDEPath + "\ProjPub\Languages" )

	' Messages this addin will hook into
	Ra3Hooks.hook1 = RA3AIH_COMMAND Or RA3AIH_CLOSE Or RA3AIH_MENUENABLE Or RA3AIM_FILEOPENED
	Ra3Hooks.hook2 = NULL
	Ra3Hooks.hook3 = NULL
	Ra3Hooks.hook4 = NULL

	Return @Ra3Hooks

End Function

' RadASM 3.x calls this function for every addin message that this addin is hooked into.
Function AddinProc Cdecl Alias "AddinProc" ( _
	ByVal hWin As HWND, _
	ByVal uMsg As UINT, _
	ByVal wParam As WPARAM, _
	ByVal lParam As LPARAM _
	) As BOOL Export

	Select Case uMsg
		Case RA3AIM_COMMAND
			Select Case LoWord( wParam )
				Case IDM_PUBLISH
					ShowProjectPublishWizard( hWin )
					'
				Case IDM_PUBLISHOPTIONS
					DialogBox( hInstance, MAKEINTRESOURCE( IDD_DLGSETTINGS ), hWin, @SettingsDialogProc )
					'
				Case RA3IDM_PROJECT_CLOSE
					SaveProjectSettings( @CurProjectFile )
					RemoveAllFolders( @ProjectSettings.Folders )
					RemoveAllFileTypes( @ProjectSettings.FileAssociations )
					CurProjectFile = ""
					CurProjectPath = ""
					'
			End Select
			'
		Case RA3AIM_FILEOPENED
			If wParam = RA3ID_PROJECT Then
				'CurProjectFile = lpRa3Data->szProjectFile
				CurProjectFile = *Cast( ZString Ptr, lParam )
				'CurProjectPath = lpRa3Data->szProjectPath
				CurProjectPath = GetDir( CurProjectFile )
				LoadProjectSettings( @CurProjectFile )
			End If
			'
		Case RA3AIM_MENUENABLE
			Dim uEnable As Integer = MF_BYCOMMAND
			If CurProjectFile <> "" Then
				uEnable Or= MF_ENABLED
			Else
				uEnable Or= MF_GRAYED
			End If
			EnableMenuItem( MakeSubMenu, IDM_PUBLISH, uEnable )
			'
		Case RA3AIM_CLOSE
			DeleteMenu( lpRa3Handles->hMenu, IDM_PUBLISH, MF_BYCOMMAND )
			DeleteMenu( lpRa3Handles->hMenu, IDM_PUBLISHOPTIONS, MF_BYCOMMAND )
			If WelcomeMsgFont Then DeleteObject( WelcomeMsgFont )
			If HeaderTitleFont Then DeleteObject( HeaderTitleFont )
			FreeLicenses
			FreeGraphics
			SaveSettingsIni( lpRa3Data->szRadASMIni )
			'
	End Select

	Return FALSE

End Function

#ElseIf Defined( ME_ADDIN )

' For MasmEdit
' Returns info on what messages the addin hooks into (in an ADDINHOOKS structure).
Function InstallAddin CDECL alias "InstallAddin" ( _
	ByVal hWin as HWND _
	) As MEHOOK Ptr Export

	' Set current IDE
	CurIDE = IDE_MASMEDIT

	' The dll's instance
	hInstance = GetModuleHandle( ExePath & "\Addins\ProjPub.dll" )
	If hInstance = NULL Then
		ErrorBox( hWin, GetTranslateString( "5009", "Can't find instance of addin." ) )
		Return @MeHooks
	End If

	' Get pointer to ADDINHANDLES, ADDINDATA and ADDINPROCS
	lpMeHandles = Cast( MEADDINHANDLES Ptr, SendMessage( hWin, MEAIM_GETHANDLES, 0,0 ) )
	lpMeData = Cast( MEADDINDATA Ptr, SendMessage( hWin, MEAIM_GETDATA, 0, 0 ) )
	lpMeFunctions = Cast( MEADDINPROCS Ptr, SendMessage( hWin, MEAIM_GETPROCS, 0, 0 ) )

	' Get IDE path
	IDEPath = lpMeData->AppPath

	'' Get a menu id
	IDM_PUBLISH = SendMessage( hWin, MEAIM_GETMENUID, 0, 0 )
	AddAccelerator( FVIRTKEY Or FNOINVERT Or FCONTROL Or FSHIFT Or FALT, VK_F5, IDM_PUBLISH )

	'' Get another menu id
	IDM_PUBLISHOPTIONS = SendMessage( hWin, MEAIM_GETMENUID, 0, 0 )

	' Load settings
	LoadSettingsRegistry( HKEY_CURRENT_USER, "Software\MasmEd1000\MasmEd" )

	' Load license, graphics and languages files
	LoadLicenses( IDEPath + "\ProjPub\Licenses" )
	LoadGraphics( IDEPath + "\ProjPub\Graphics" )
	LoadLanguages( IDEPath + "\ProjPub\Languages" )

	' Messages this addin will hook into
	MeHooks.hook1 = MEAIH_COMMAND Or MEAIH_CLOSE Or MEAIH_MENUUPDATE Or MEAIM_FILEOPEN
	MeHooks.hook2 = NULL
	MeHooks.hook3 = NULL
	MeHooks.hook4 = NULL

	Return @MeHooks

End Function

' MasmEdit calls this function for every addin message that this addin is hooked into.
Function AddinProc Cdecl Alias "AddinProc" ( _
	ByVal hWin As HWND, _
	ByVal uMsg As UINT, _
	ByVal wParam As WPARAM, _
	ByVal lParam As LPARAM _
	) As BOOL Export

	Select Case uMsg
		Case MEAIM_COMMAND
			Select Case LoWord( wParam )
				Case IDM_PUBLISH
					ShowProjectPublishWizard( hWin )
					'
				Case IDM_PUBLISHOPTIONS
					DialogBox( hInstance, MAKEINTRESOURCE( IDD_DLGSETTINGS ), hWin, @SettingsDialogProc )
					'
				Case MEIDM_PROJECT_CLOSE
					SaveProjectSettings( @CurProjectFile )
					RemoveAllFolders( @ProjectSettings.Folders )
					RemoveAllFileTypes( @ProjectSettings.FileAssociations )
					CurProjectFile = ""
					CurProjectPath = ""
					'
			End Select
			'
		Case MEAIM_FILEOPEN
			If wParam = MEIDC_MES Then
				CurProjectFile = *Cast( ZString Ptr, lParam )
				CurProjectPath = GetDir( CurProjectFile )
				LoadProjectSettings( @CurProjectFile )
			End If
			'
		Case MEAIM_MENUUPDATE
			'' Fill a MENUITEMINFO structure 
			Dim mii As MENUITEMINFO
			mii.cbSize = SizeOf( MENUITEMINFO )
			mii.fMask  = MIIM_SUBMENU
		
			'' Get handle to 'Make' popup
			GetMenuItemInfo( lpMeHandles->hMnu, MEIDM_MAKE, FALSE, @mii )
		
			MakeSubMenu = mii.hSubMenu
		
			'' Add our menu item to Tools menu
			Dim uEnable As Integer = MF_BYCOMMAND
			If CurProjectFile <> "" Then
				uEnable Or= MF_ENABLED
			Else
				uEnable Or= MF_GRAYED
			End If
			AppendMenu( MakeSubMenu, MF_STRING Or uEnable, IDM_PUBLISH, GetTranslateString( "10000", "Publish" ) & !"\tCtrl+Shift+Alt+F5" )
		
			'' Get handle to 'Options' popup
			GetMenuItemInfo( lpMeHandles->hMnu, MEIDM_OPTIONS, FALSE, @mii )
		
			'' Add our menu item to Tools menu
			AppendMenu( mii.hSubMenu, MF_STRING, IDM_PUBLISHOPTIONS, GetTranslateString( "10001", "Project Publisher" ) )
			'
		Case MEAIM_CLOSE
			DeleteMenu( lpMeHandles->hMnu, IDM_PUBLISH, MF_BYCOMMAND )
			DeleteMenu( lpMeHandles->hMnu, IDM_PUBLISHOPTIONS, MF_BYCOMMAND )
			If WelcomeMsgFont Then DeleteObject( WelcomeMsgFont )
			If HeaderTitleFont Then DeleteObject( HeaderTitleFont )
			FreeLicenses
			FreeGraphics
			SaveSettingsRegistry( HKEY_CURRENT_USER, "Software\MasmEd1000\MasmEd" )
			'
	End Select

	Return FALSE

End Function

#EndIf


Sub DisableUnneededFeatures( _
	)

	DisabledFeatures = NULL
	Select Case ProjectSettings.ProjectType
		Case PROJPUB_PROJTYPE_BASIC
			DisabledFeatures Or= _
				PROJPUB_FEATURES_REMDRVINST Or _
				PROJPUB_FEATURES_FILEASSOC Or _
				PROJPUB_FEATURES_OPENWITH
			'
		Case PROJPUB_PROJTYPE_GAME
			DisabledFeatures Or= _
				PROJPUB_FEATURES_REMDRVINST Or _
				PROJPUB_FEATURES_FILEASSOC Or _
				PROJPUB_FEATURES_OPENWITH
			'
		Case PROJPUB_PROJTYPE_FBEADDIN
			DisabledFeatures Or=  _
				PROJPUB_FEATURES_SHORTCUTS Or  _
				PROJPUB_FEATURES_FILEASSOC Or _
				PROJPUB_FEATURES_RUNONFINISH Or _
				PROJPUB_FEATURES_OPENWITH
			'
		Case PROJPUB_PROJTYPE_RAADDIN
			DisabledFeatures Or=  _
				PROJPUB_FEATURES_SHORTCUTS Or  _
				PROJPUB_FEATURES_FILEASSOC Or _
				PROJPUB_FEATURES_RUNONFINISH Or _
				PROJPUB_FEATURES_OPENWITH
			'
		Case PROJPUB_PROJTYPE_MEADDIN
			DisabledFeatures Or=  _
				PROJPUB_FEATURES_SHORTCUTS Or  _
				PROJPUB_FEATURES_FILEASSOC Or _
				PROJPUB_FEATURES_RUNONFINISH Or _
				PROJPUB_FEATURES_OPENWITH
			'
		Case PROJPUB_PROJTYPE_ADVANCE
			'
		Case PROJPUB_PROJTYPE_GTKP
			'
		Case PROJPUB_PROJTYPE_APPPLUGIN
			DisabledFeatures Or=  _
				PROJPUB_FEATURES_REMDRVINST Or  _
				PROJPUB_FEATURES_SHORTCUTS Or  _
				PROJPUB_FEATURES_FILEASSOC Or _
				PROJPUB_FEATURES_RUNONFINISH Or _
				PROJPUB_FEATURES_OPENWITH
			'
	End Select

End Sub


'' Get translation of strings.
Function GetTranslateString( _
	ByRef KeyName As Const String, _
	ByRef KeyValueDefault As Const String _
	) As String

	If CurIDE = IDE_FBEDIT Then
		'' Get translation
		Dim KeyValue As String = lpFbEFunctions->FindString( _
			lpFbEData->hLangMem, _
			ADDIN_NAME, _
			KeyName _
			)
		'' Check if valid
		If KeyValue <> "" Then
			Return KeyValue
		Else
			Return KeyValueDefault
		End If
	ElseIf CurIDE = IDE_RADASM2 Then
		Dim KeyNameWide As WString Ptr = Cast( WString Ptr, New UByte [(Len( KeyName )+1)*2] )
		*KeyNameWide = KeyName
		Dim KeyValueDefaultWide As WString Ptr = Cast( WString Ptr, New UByte [(Len( KeyValueDefault )+1)*2] )
		*KeyValueDefaultWide = KeyValueDefault
		Dim WideBuffer As WString * MAX_PATH+1
		lpRa2Functions->GetLangString( KeyNameWide, KeyValueDefaultWide, @WideBuffer, MAX_PATH )
		Delete[] KeyNameWide
		Delete[] KeyValueDefaultWide
		If WideBuffer <> "" Then
			Return WideBuffer
		Else
			Return KeyValueDefault
		End If
	Else
		Return KeyValueDefault
	End If

End Function


Sub TranslateAddinDialog( _
	ByVal hWin As HWND _
	)

	Select Case CurIDE
		Case IDE_FBEDIT
			lpFbEFunctions->TranslateAddinDialog( hWin, ADDIN_NAME )
			'
	End Select

End Sub


Sub AddAccelerator( _
	ByVal fvirt As Integer, _
	ByVal akey As Integer, _
	ByVal id As Integer _
	)

	Dim Found As Integer
	Dim acl(500) As ACCEL
	Dim nAccel As Integer
	Dim IDEAccel As HACCEL

	Select Case CurIDE
		Case IDE_FBEDIT
			IDEAccel = lpFbEHandles->haccel
			'
		Case IDE_RADASM2 
			IDEAccel = lpRa2Handles->haccel
			'
		Case IDE_RADASM3 
			IDEAccel = lpRa3Handles->hAccel
			'
		Case IDE_MASMEDIT 
			IDEAccel = lpMeHandles->hAccel
			'
	End Select

	nAccel = CopyAcceleratorTable( IDEAccel, NULL, 0 )
	CopyAcceleratorTable( IDEAccel, @acl(0), nAccel )
	DestroyAcceleratorTable( IDEAccel )

	' Check if id or accelerator exist
	For i As Integer = 0 To nAccel-1
		If acl(i).cmd = id Then
			' id exist, update accelerator
			acl(i).fVirt = fvirt
			acl(i).key = akey
			Found = -1
			Exit For
		ElseIf acl(i).fVirt = fvirt And _
			acl(i).key = akey Then
			' Accelerator exist, update id
			acl(i).cmd = id
			Found = -1
			Exit For
		End If
	Next

	If Not Found Then
		' Add new accelerator
		acl(nAccel).fVirt = fvirt
		acl(nAccel).key = akey
		acl(nAccel).cmd = id
		nAccel += 1
	End If

	Select Case CurIDE
		Case IDE_FBEDIT
			lpFbEHandles->haccel = CreateAcceleratorTable( @acl(0), nAccel )
			'
		Case IDE_RADASM2
			lpRa2Handles->haccel = CreateAcceleratorTable( @acl(0), nAccel )
			'
		Case IDE_RADASM3
			lpRa3Handles->hAccel = CreateAcceleratorTable( @acl(0), nAccel )
			'
		Case IDE_MASMEDIT
			lpMeHandles->hAccel = CreateAcceleratorTable( @acl(0), nAccel )
			'
	End Select

End Sub


Function OpenFileName( _
	ByVal hWin As HWND, _
	ByVal lpszTitle As ZString Ptr, _
	ByVal lpszInitialDir As ZString Ptr, _
	ByVal lpszFilter As ZString Ptr _
	) As String

	Dim ofn As OPENFILENAME
	Dim buff As ZString * MAX_PATH+1

	With ofn
		.lStructSize = SizeOf(OPENFILENAME)
		.hwndOwner = hWin
		.hInstance = hInstance
		.lpstrInitialDir = lpszInitialDir
		.lpstrFile = @buff
		.nMaxFile = MAX_PATH
		.lpstrFilter = lpszFilter
		.lpstrTitle = lpszTitle
		.Flags = OFN_PATHMUSTEXIST Or OFN_HIDEREADONLY Or OFN_EXPLORER
	End With

	GetOpenFileName( @ofn )

	Return buff

End Function


Function EnableDlgItem( _
	ByVal hWin As HWND, _
	ByVal nIDDlgItem As Integer, _
	ByVal bEnable As BOOL _
	) As BOOL

	Return EnableWindow( GetDlgItem( hWin, nIDDlgItem ), bEnable ) 

End Function


Function Replace( _
	ByRef InputString As Const String, _
	ByRef SearchString As Const String, _
	ByRef Replacement As Const String _
	) As String

	Dim As Integer x, y = 1
	Dim Temp As String = InputString
	Dim RetStr As String

	Do While y
		x = InStr( Temp, SearchString )
		If x Then
			Print SearchString & " is found at " & x
			RetStr &= Left( Temp, x-1 ) + Replacement
			Print "RetStr is now " & RetStr
			Temp = Mid( Temp, x + Len( SearchString ) )
			Print
			Continue Do
		End If
		y = 0
	Loop

	If Temp <> "" And RetStr <> "" Then
		Return RetStr & Temp
	ElseIf RetStr <> "" Then
		Return RetStr
	Else
		Return InputString
	End If

End Function


Function EscapeNSIS( _
	ByRef UnescapedStr As Const String _
	) As String
	
	Dim As Integer DollarSignPos, DoubleQuotePos
	Dim y As Integer = 1
	Dim TempStr As String = UnescapedStr
	Dim RetString As String

	Print "UnescapedStr is " & UnescapedStr

	'TempStr = Replace( TempStr, "$", "$\$" )
	TempStr = Replace( TempStr, """", "$\""" )

	Return TempStr

End Function


Function FilterNSISIdentifier( _
	ByRef NSISIdentifier As Const String _
	) As String

	Dim TempString As String = NSISIdentifier
	Dim RetString As String
	Dim Char As Unsigned Integer

	Do While TempString <> ""
		Char = Asc( TempString )
		If (Char >= &h41 And Char <= &h5A) Or _	' A-Z
			(Char >= &h61 And Char <= &h7A) Or _	' a-z
			(Char >= &h30 And Char <= &h39) Or _	' 0-9
			Char = &h5F Then _							' _
			RetString &= Chr( Char )
		TempString = Mid( TempString, 2 )
	Loop

	Return RetString

End Function

